/*ckwg +29
 * Copyright 2014-2015 by Kitware, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  * Neither name of Kitware, Inc. nor the names of any contributors may be used
 *    to endorse or promote products derived from this software without specific
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file
 * \brief VisCL algorithm registration function implementation
 */

#include "register_algorithms.h"

#include <arrows/algorithm_plugin_interface_macros.h>
#include <arrows/viscl/convert_image.h>
#include <arrows/viscl/detect_features.h>
#include <arrows/viscl/extract_descriptors.h>
#include <arrows/viscl/match_features.h>
#include <vital/registrar.h>


namespace kwiver {
namespace arrows {
namespace vcl {

/// Register VisCL algorithm implementations with the given or global registrar
int register_algorithms( vital::registrar &reg )
{
  REGISTRATION_INIT( reg );

  REGISTER_TYPE( vcl::convert_image );
  REGISTER_TYPE( vcl::detect_features );
  REGISTER_TYPE( vcl::extract_descriptors );
  REGISTER_TYPE( vcl::match_features );

  REGISTRATION_SUMMARY();
  return REGISTRATION_FAILURES();
}

} // end namespace vcl
} // end namespace arrows
} // end namespace kwiver
