/*ckwg +29
 * Copyright 2011-2012 by Kitware, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  * Neither name of Kitware, Inc. nor the names of any contributors may be used
 *    to endorse or promote products derived from this software without specific
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "registration.h"

#include "sync_scheduler.h"
#include "thread_per_process_scheduler.h"
#include "thread_pool_scheduler.h"

#include <sprokit/pipeline/scheduler_registry.h>

/**
 * \file examples/registration.cxx
 *
 * \brief Register schedulers for use.
 */

using namespace sprokit;

void
register_schedulers()
{
  static scheduler_registry::module_t const module_name = scheduler_registry::module_t("example_schedulers");

  scheduler_registry_t const registry = scheduler_registry::self();

  if (registry->is_module_loaded(module_name))
  {
    return;
  }

  registry->register_scheduler("sync", "Run the pipeline synchronously", create_scheduler<sync_scheduler>);
  registry->register_scheduler("thread_per_process", "Run each process in its own thread", create_scheduler<thread_per_process_scheduler>);
  registry->register_scheduler("thread_pool", "Use a pool of threads to step processes", create_scheduler<thread_pool_scheduler>);

  registry->mark_module_as_loaded(module_name);
}
