/*ckwg +29
 * Copyright 2011-2012 by Kitware, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  * Neither name of Kitware, Inc. nor the names of any contributors may be used
 *    to endorse or promote products derived from this software without specific
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SPROKIT_PIPELINE_MODULES_H
#define SPROKIT_PIPELINE_MODULES_H

#include "pipeline-config.h"
#include <sprokit/pipeline_util/path.h>

#include <vector>

/**
 * \file modules.h
 *
 * \brief Functions dealing with modules in sprokit.
 */

namespace sprokit
{

typedef path_t::string_type module_path_t;
typedef std::vector<module_path_t> module_paths_t;

/**
 * \brief Get module load path.
 *
 * This function returns the module load path as a list of
 * directories.
 *
 * @return List of directories for loading modules.
 */
SPROKIT_PIPELINE_EXPORT sprokit::module_paths_t get_module_load_path();


/**
 * \brief Load modules from the system path.
 *
 * This function loads all known modules. That is, all modules that
 * can be found. Modules are shared objects or DLL's that are in one
 * of the search paths and have a register_processes() symbol
 * available.
 *
 * The default module path is based on where sprokit will be installed
 * and points to the \b sprokit subdirectory of the installed libraries.
 *
 * Additional modules can be fetched from a path specified in the
 * environment \b SPROKIT_MODULE_PATH.
 */
SPROKIT_PIPELINE_EXPORT void load_known_modules();

}

#endif // SPROKIT_PIPELINE_MODULES_H
