/*ckwg +5
 * Copyright 2012-2016 by Kitware, Inc. All Rights Reserved. Please refer to
 * KITWARE_LICENSE.TXT for licensing information, or contact General Counsel,
 * Kitware, Inc., 28 Corporate Drive, Clifton Park, NY 12065.
 */

#include "file_format_base.h"

#include <vul/vul_file.h>
#include <vul/vul_reg_exp.h>
#include <vul/vul_string.h>

#include <track_oracle/schema_algorithm.h>

#include <vital/logger/logger.h>
static kwiver::vital::logger_handle_t main_logger( kwiver::vital::get_logger( __FILE__ ) );

using std::istream;
using std::map;
using std::ostream;
using std::string;
using std::vector;

namespace // anon
{

vul_reg_exp
glob_to_regexp( const string& glob )
{
  //
  // This code is lifted straight from vul_file_iterator.cxx.
  //

  string baseglob = vul_file::basename(glob);
  string::iterator i = baseglob.begin();
  bool prev_slash=false, in_sqr_brackets=false;
  //assemble the Regexp string
  string re = "^"; // match the start of the string
  while (i != baseglob.end())
  {
    if (*i=='\\' && !prev_slash)
      prev_slash = true;
    else if (prev_slash)
    {
      prev_slash = false;
      re.append(1,('\\'));
      re.append(1,*i);
    }
    else if (*i=='[' && !in_sqr_brackets)
    {
      in_sqr_brackets = true;
      re.append(1,'[');
    }
    else if (*i==']' && in_sqr_brackets)
    {
      in_sqr_brackets = false;
      re.append(1,']');
    }
    else if (*i=='?' && !in_sqr_brackets)
      re.append(1,'.');
    else if (*i=='*' && !in_sqr_brackets)
      re.append(".*");
    else
      re.append(vul_reg_exp::protect(*i));

    ++i;
  }
  // match the end of the string
  re += '$';

  return vul_reg_exp( re.c_str() );

}

} // anon

namespace kwiver {
namespace track_oracle {

file_format_base
::file_format_base( file_format_enum fmt,
                    const string& desc
  )
  : type( fmt ), description( desc )
{
}

file_format_base
::~file_format_base()
{
}

string
file_format_base
::format_description() const
{
  return this->description;
}

void
file_format_base
::set_format_description( const string& d )
{
  this->description = d;
}

file_format_enum
file_format_base
::get_format() const
{
  return this->type;
}


vector< string >
file_format_base
::format_globs() const
{
  return this->globs;
}

bool
file_format_base
::filename_matches_globs( string fn ) const
{
  vul_string_downcase( fn );
  for (size_t i=0; i<this->globs.size(); ++i)
  {
    vul_reg_exp r = glob_to_regexp( this->globs[i] );
    if (r.find( fn )) return true;
  }
  return false;
}

map< field_handle_type, track_base_impl::schema_position_type >
file_format_base
::enumerate_schema() const
{
  track_base_impl* t = this->schema_instance();
  map< field_handle_type, track_base_impl::schema_position_type > ret = t->list_schema_elements();
  delete t;
  return ret;
}

file_format_reader_opts_base&
file_format_base
::options()
{
  return this->null_opts;
}

bool
file_format_base
::read( istream&,
        track_handle_list_type& ) const
{
  // by default, not implemented
  LOG_ERROR( main_logger, "Stream reading not supported for " << file_format_type::to_string( this->type ) << " tracks" );
  return false;
}

bool
file_format_base
::write( const string&,
         const track_handle_list_type& ) const
{
  // by default, not implemented
  LOG_ERROR( main_logger, "File writing not supported for " << file_format_type::to_string( this->type ) << " tracks" );
  return false;
}

bool
file_format_base
::write( ostream&,
         const track_handle_list_type& ) const
{
  // by default, not implemented
  LOG_ERROR( main_logger, "Stream writing not supported for " << file_format_type::to_string( this->type ) << " tracks" );
  return false;
}

bool
file_format_base
::read( const string& fn,
        track_handle_list_type& tracks,
        const track_base_impl& required_fields,
        vector< element_descriptor >& missing_fields ) const
{
  if (! this->read( fn, tracks )) return false;
  missing_fields = schema_algorithm::name_missing_fields( required_fields, tracks );
  return true;
}

bool
file_format_base
::read( istream& is,
        track_handle_list_type& tracks,
        const track_base_impl& required_fields,
        vector< element_descriptor >& missing_fields ) const
{
  if (! this->read( is, tracks )) return false;
  missing_fields = schema_algorithm::name_missing_fields( required_fields, tracks );
  return true;
}

} // ...track_oracle
} // ...kwiver
